//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* PWS EQ.cpp
 * 
 * A VST equalizer.
 * It's a 32-band parametric EQ, that can be used as a graphic EQ as well.
 *
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 * 
 * Contents:
     1. Constructor and destructor
     2. canDo and product info
     3. Input and output properties
     4. Settings for audio processing
     5. Parameters
     6. Programs
     7. Midi
     8. Main process
 */

#include "PWS EQ.h"

namespace PWSPlugCanDos
{
    const char* canDoSendVstEvents = "sendVstEvents"; ///< plug-in will send Vst events to Host
    const char* canDoSendVstMidiEvent = "sendVstMidiEvent"; ///< plug-in will send MIDI events to Host
    const char* canDoReceiveVstEvents = "receiveVstEvents"; ///< plug-in can receive MIDI events from Host
    const char* canDoReceiveVstMidiEvent = "receiveVstMidiEvent"; ///< plug-in can receive MIDI events from Host 
    const char* canDoReceiveVstTimeInfo = "receiveVstTimeInfo"; ///< plug-in can receive Time info from Host 
    const char* canDoOffline = "offline"; ///< plug-in supports offline functions (#offlineNotify, #offlinePrepare, #offlineRun)
    const char* canDoMidiProgramNames = "midiProgramNames"; ///< plug-in supports function #getMidiProgramName ()
    const char* canDoBypass = "bypass"; ///< plug-in supports function #setBypass ()
}

//-----------------------------------------------------------------------------
// 1. Constructor and destructor
//-----------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
       return new PWSEQ (audioMaster);
}

//-----------------------------------------------------------------------------
PWSEQ::PWSEQ (audioMasterCallback audioMaster)
: AudioEffectX (audioMaster, kNumPrograms, kNumParams)
{
    EditorLoaded = false;
    if (audioMaster)
    {
        setNumInputs (kNumInputs);              //no inputs
        setNumOutputs (kNumOutputs);            //2 outputs
        canProcessReplacing ();                 //can use processReplacing ()
        isSynth (false);                        //is a synth     
        setUniqueID (CCONST('6','5','2','5'));  //MUST be unique
    }
    
    fs = 44100.0f;                   //default
    
    f = new float[kNumBands];
    b = new float[kNumBands];
    p = new float[kNumBands];
    
    params = new float[kNumParams];
     
    EQ = new peakinghandler(kNumBands, f, b, p);

    initProgs();
    
    ev = NULL;

    ByPass = false;

    AQGUI* AQEditor = new AQGUI (this);
    setEditor (AQEditor);
    EditorLoaded = true;

	dbRange = 12.0f;

	GUIChanges = false;
}

//-----------------------------------------------------------------------------
PWSEQ::~PWSEQ ()
{
    if (f) delete[] f;
    if (b) delete[] b;
    if (p) delete[] p;
    if (params) delete[] params;
    
    if (EQ) delete EQ;

    //AQGUI* AQEditor = (AQGUI*) getEditor();
    //if (AQEditor) delete AQEditor;
}

//-----------------------------------------------------------------------------
// 2. canDo and product info
//-----------------------------------------------------------------------------
VstInt32 PWSEQ::canDo (char* text)
{
    if ((!strcmp (text, PWSPlugCanDos::canDoSendVstEvents)))
        return -1;
    if ((!strcmp (text, PWSPlugCanDos::canDoSendVstMidiEvent)))
        return -1;
    if ((!strcmp (text, PWSPlugCanDos::canDoReceiveVstEvents)))
        return 1;
    if ((!strcmp (text, PWSPlugCanDos::canDoReceiveVstMidiEvent)))
        return 1;
    if ((!strcmp (text, PWSPlugCanDos::canDoReceiveVstTimeInfo)))
        return 1;
    if ((!strcmp (text, PWSPlugCanDos::canDoOffline)))
        return -1;
    if ((!strcmp (text, PWSPlugCanDos::canDoMidiProgramNames)))
        return -1;
    if ((!strcmp (text, PWSPlugCanDos::canDoBypass)))
        return 1;
    return -1;
}

//-----------------------------------------------------------------------------
bool PWSEQ::getEffectName (char* name)
{
    vst_strncpy(name, "Aqualizer", kVstMaxEffectNameLen);
    return true;
}

//-----------------------------------------------------------------------------
bool PWSEQ::getVendorString (char* text)
{
    vst_strncpy (text, "Jelle and Josse VST", kVstMaxVendorStrLen);
    return true;
}

//-----------------------------------------------------------------------------
bool PWSEQ::getProductString (char* text)
{
    vst_strncpy (text, "Aqualizer", kVstMaxProductStrLen);
    return true;
}

//-----------------------------------------------------------------------------
VstInt32 PWSEQ::getVendorVersion ()
{
    return 1100;
}

//-----------------------------------------------------------------------------
// 3. Input and Output properties
//-----------------------------------------------------------------------------
bool PWSEQ::getInputProperties (VstInt32 index, VstPinProperties* properties)
{
    if (index < kNumInputs)
    {
        sprintf_s (properties->label, "Aqualizer in %1d", index + 1);
        properties->arrangementType = kSpeakerArrStereo;
        properties->flags = kVstPinIsActive | kVstPinIsStereo | kVstPinUseSpeaker;
        return true;
    }
    return false;
}

//-----------------------------------------------------------------------------
bool PWSEQ::getOutputProperties (VstInt32 index, VstPinProperties* properties)
{
    if (index < kNumOutputs)
    {
            sprintf_s (properties->label, "Aqualizer out %1d", index + 1);
            properties->arrangementType = kSpeakerArrStereo;
            properties->flags = kVstPinIsActive | kVstPinIsStereo | kVstPinUseSpeaker;
            return true;
    }
    return false;
}

//-----------------------------------------------------------------------------
bool PWSEQ::setBypass (bool OnOff) 
{
    ByPass = OnOff;
    return true;
}

//-----------------------------------------------------------------------------
// 4. Settings for audio processing
//-----------------------------------------------------------------------------
void PWSEQ::setSampleRate (float sampleRate)
{
    AudioEffectX::setSampleRate (sampleRate);
    for (int i = 0; i < kNumBands; i++)
    {
        setParameter(kFreq1 + i, f[i] * 2.0f * fs / sampleRate);
        ((AEffGUIEditor*)editor)->setParameter (kFreq1 + i, 0.0f);
    }
    fs = sampleRate;
}

//-----------------------------------------------------------------------------
void PWSEQ::setBlockSize (VstInt32 blockSize)
{
    bs = blockSize;
}

//-----------------------------------------------------------------------------
// 5. Parameters
//-----------------------------------------------------------------------------
void PWSEQ::setParameter (VstInt32 index, float value)
{
    if (index < 0 || index >= kNumParams) return;
    
    params[index] = value;
	if (EditorLoaded && !GUIChanges)
		((AEffGUIEditor*)editor)->setParameter (index, value);
    
    // Check if one of the bands has been changed (peak)
    int i = index - kBand1;
    if (i < kNumBands)
    {
        float dB = (value - 0.5f) * 2 * dbRange;   //spread them on a range from -12 to +12 (dB)
        p[i] = pow (10.0f, dB / 10.0f);            //inverse dB calculation; calculate actual peak value: 10^(peak/10)
        EQ->peakChanged(i);
		if (!settingProgram)
			(progs[curProg]).bandGain[i] = dB;
        return;
    }
    
    // Check if one of the frequencies has been changed
    i = index - kFreq1;
    if (i < kNumBands)
    {
        f[i] = value / 2.0f;
        EQ->freqChanged(i);
		if (!settingProgram)
			(progs[curProg]).freqs[i] = f[i] * fs;
        return;
    }
    
    // Check if one of the bandwidths has been changed
    i = index - kBW1;
    if (i < kNumBands)
    {
        if (value < 0.00001f) value = 0.00001f;
        b[i] = value;                 //minimum: 0.00001, maximum: 1
        EQ->BWChanged(i);
		if (!settingProgram)
			(progs[curProg]).BWs[i] = b[i];
        return;
    }
    
    // Check other stuff
    if (index == kNumBandsActive)
    {
        numBandsActive = (int) (0.5f + ceil (value * (float) kNumBands));
        // take the ceil value and make it an int with this little trick
        if (numBandsActive == 0) numBandsActive = 1;
        EQ->setNumActive(numBandsActive);
		if (!settingProgram)
			(progs[curProg]).BandsActive = numBandsActive;
        return;
    }
    if (index == kLinkBW)
    {
        int l = (int) (0.5f + floor (value * 2.0f));
        if (l == 0) BWlinked = false;
        else BWlinked = true;
        EQ->linkBW(BWlinked);
		if (!settingProgram)
			(progs[curProg]).BWlink = BWlinked;
        return;
    }
    if (index == kBW)
    {
        if (value < 0.00001f) value = 0.00001f;
        bw = value;
        EQ->setBW(value);
		if (!settingProgram)
			(progs[curProg]).BW = bw;
        return;
    }
    if (index == kGain)
    {
        float dB = (value - 0.5f) * 2 * dbRange; //spread them on a range from -12 to +12 (dB)
        gain = pow (10.0f, dB / 10.0f);          //inverse dB calculation; calculate actual peak value: 10^(peak/10)
		if (!settingProgram)
			(progs[curProg]).mainGain = dB;
        return;
    }
	if (index == kMinFreq)
	{
		float fr = value / 2.0f;
		minFreq = fr * fs;
		if (!settingProgram)
			(progs[curProg]).freqMin = minFreq;
		return;
	}
	if (index == kMaxFreq)
	{
		float fr = value / 2.0f;
		maxFreq = fr * fs;
		if (!settingProgram)
			(progs[curProg]).freqMax = maxFreq;
		return;
	}
}

//-----------------------------------------------------------------------------
float PWSEQ::getParameter (VstInt32 index)
{
    if (index < 0 || index >= kNumParams) return 0.0f;
    
    return params[index];
}

//-----------------------------------------------------------------------------
void PWSEQ::getParameterLabel (VstInt32 index, char* label)
{
    if (index < 0 || index >= kNumParams) return;
    
    // Check if one of the bands has been asked (peak)
    int i = index - kBand1;
    if (i < kNumBands)
    {
        vst_strncpy (label, "dB", kVstMaxParamStrLen);
        return;
    }
    
    // Check if one of the frequencies has been asked
    i = index - kFreq1;
    if (i < kNumBands)
    {
        vst_strncpy (label, "Hz", kVstMaxParamStrLen);
        return;
    }
    
    // Check if one of the bandwidths has been asked
    i = index - kBW1;
    if (i < kNumBands || index == kBW)
    {
        vst_strncpy (label, " ", kVstMaxParamStrLen);
        return;
    }
    
    // Check other stuff
    if (index == kNumBandsActive)
    {
        vst_strncpy (label, " ", kVstMaxParamStrLen);
        return;
    }
    if (index == kLinkBW)
    {
        vst_strncpy (label, "On/Off", kVstMaxParamStrLen);
        return;
    }
    if (index == kGain)
    {
        vst_strncpy (label, "dB", kVstMaxParamStrLen);
        return;
    }
	if (index == kMinFreq)
	{
		vst_strncpy (label, "Hz", kVstMaxParamStrLen);
		return;
	}
	if (index == kMaxFreq)
	{
		vst_strncpy (label, "Hz", kVstMaxParamStrLen);
		return;
	}
}

//-----------------------------------------------------------------------------
void PWSEQ::getParameterDisplay (VstInt32 index, char* text)
{
    if (index < 0 || index >= kNumParams) return;
    
    // Check if one of the bands has been asked (peak)
    int i = index - kBand1;
    if (i < kNumBands)
    {
        float2string ((params[i] - 0.5f) * 2 * dbRange, text, kVstMaxParamStrLen);
        return;
    }
    
    // Check if one of the frequencies has been asked
    i = index - kFreq1;
    if (i < kNumBands)
    {
        float2string (f[i] * fs, text, kVstMaxParamStrLen);
        return;
    }
    
    // Check if one of the bandwidths has been asked
    i = index - kBW1;
    if (i < kNumBands)
    {
        float2string (b[i], text, kVstMaxParamStrLen);
        return;
    }
    
    // Check other stuff
    if (index == kNumBandsActive)
    {
        sprintf (text, "%i", numBandsActive);
        return;
    }
    if (index == kLinkBW)
    {
        if (BWlinked)   vst_strncpy (text, "On", kVstMaxParamStrLen);
        else            vst_strncpy (text, "Off", kVstMaxParamStrLen);
        return;
    }
    if (index == kBW)
    {
        float2string (bw, text, kVstMaxParamStrLen);
        return;
    }
    if (index == kGain)
    {
        float2string ((params[i] - 0.5f) * 2 * dbRange, text, kVstMaxParamStrLen);
        return;
    }
	if (index == kMinFreq)
	{
		float2string (minFreq * fs, text, kVstMaxParamStrLen);
		return;
	}
	if (index == kMaxFreq)
	{
		float2string (maxFreq * fs, text, kVstMaxParamStrLen);
		return;
	}
}

//-----------------------------------------------------------------------------
void PWSEQ::getParameterName (VstInt32 index, char* text)
{
    if (index < 0 || index >= kNumParams) return;
    
    // Check if one of the bands has been asked (peak)
    int i = index - kBand1;
    if (i < kNumBands)
    {
        sprintf_s (text, kVstMaxParamStrLen, "Band %d", i + 1);
        return;
    }
    
    // Check if one of the frequencies has been asked
    i = index - kFreq1;
    if (i < kNumBands)
    {
        sprintf_s (text, kVstMaxParamStrLen, "Freq %d", i + 1);
        return;
    }
    
    // Check if one of the bandwidths has been asked
    i = index - kBW1;
    if (i < kNumBands)
    {
        sprintf_s (text, kVstMaxParamStrLen, "BW %d", i + 1);
        return;
    }
    
    // Check other stuff
    if (index == kNumBandsActive)
    {
        vst_strncpy (text, "Bands", kVstMaxParamStrLen);
        return;
    }
    if (index == kLinkBW)
    {
        vst_strncpy (text, "Link BW", kVstMaxParamStrLen);
        return;
    }
    if (index == kBW)
    {
        vst_strncpy (text, "BW (all)", kVstMaxParamStrLen);
        return;
    }
    if (index == kGain)
    {
        vst_strncpy (text, "Gain", kVstMaxParamStrLen);
        return;
    }
	if (index == kMinFreq)
	{
		vst_strncpy (text, "fMin", kVstMaxParamStrLen);
		return;
	}
	if (index == kMaxFreq)
	{
		vst_strncpy (text, "fMax", kVstMaxParamStrLen);
		return;
	}
}

//-----------------------------------------------------------------------------
VstInt32 PWSEQ::getNumBandsActive ()
{
    return numBandsActive;
}

//-----------------------------------------------------------------------------
bool PWSEQ::getBWLinked ()
{
    return BWlinked;
}

//-----------------------------------------------------------------------------
void PWSEQ::calcMinMaxFreqs ()
{
    EQ->logSpread (minFreq / fs, maxFreq / fs);
    for (int i = 0; i < numBandsActive; i++)
    {
		GUIChanges = false;
        setParameter (kFreq1 + i, f[i] * 2.0f);    //value is ignored
		GUIChanges = true;
    }
}

//-----------------------------------------------------------------------------
void PWSEQ::setGUIChanges (bool ch)
{
	GUIChanges = ch;
}

//-----------------------------------------------------------------------------
// 6. Programs
//-----------------------------------------------------------------------------
void PWSEQ::initProgs ()
{    
    //progs = new programs[kNumPrograms];
    //for (int i = 0; i < kNumPrograms; i++)
    //    (progs[i]).name = new char[kVstMaxProgNameLen];
    
    int kMaxLen = min(kVstMaxProgNameLen, 24);
    {
        //program 1: F:[20,25,31.5,40,50,63,80,100,125,160,200,250,315,400,500,630,800,1000,
        //             1250,1600,2000,2500,3150,4000,5000,6300,8000,10000,12500,16000,20000]/fs, BW: .1
        (progs[0]).BandsActive = 31;
        (progs[0]).BWlink = true;                   //BWs[] will be ignored
        (progs[0]).BW = 0.1f;
        
        float fr[] = {20,25,31.5,40,50,63,80,100,125,160,200,250,315,400,500,630,800,1000,1250,1600,
               2000,2500,3150,4000,5000,6300,8000,10000,12500,16000,20000};
        for (int i = 0; i < (progs[0]).BandsActive; i++)
            (progs[0]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[0]).name, "31 Band Graphic EQ", kMaxLen);
    }
    {
        //program 2: F:[25,40,63,100,160,250,400,630,1000,1600,2500,4000,6300,10000,16000]/fs, BW: .2
        (progs[1]).BandsActive = 15;
        (progs[1]).BWlink = true;                   //BWs[] will be ignored
        (progs[1]).BW = 0.2f;
        
        float fr[] = {25,40,63,100,160,250,400,630,1000,1600,2500,4000,6300,10000,16000};
        for (int i = 0; i < (progs[1]).BandsActive; i++)
            (progs[1]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[1]).name, "15 Band Graphic EQ", kMaxLen);
    }
    {
        //program 3: F:[31.25,62.5,125,250,500,1000,2000,4000,8000,16000]/fs, BW: .3
        (progs[2]).BandsActive = 10;
        (progs[2]).BWlink = true;                   //BWs[] will be ignored
        (progs[2]).BW = 0.3f;
        
        float fr[] = {31.25,62.5,125,250,500,1000,2000,4000,8000,16000};
        for (int i = 0; i < (progs[2]).BandsActive; i++)
            (progs[2]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[2]).name, "10 Band Graphic EQ", kMaxLen);
    }
    {
        //program 4: F:[80,2500,12000]/fs, BW: 1
        (progs[3]).BandsActive = 3;
        (progs[3]).BWlink = true;                   //BWs[] will be ignored
        (progs[3]).BW = 1.0f;
        
        float fr[] = {80,2500,12000};
        for (int i = 0; i < (progs[3]).BandsActive; i++)
            (progs[3]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[3]).name, "3 Band DJ EQ", kMaxLen);
    }
    {
        //program 5: F:[32,64,128,256,512,1024,2048,4096,8192,16384]
        (progs[4]).BandsActive = 10;
        (progs[4]).BWlink = true;                   //BWs[] will be ignored
        (progs[4]).BW = .3f;

        float fr[] = {32,64,128,256,512,1024,2048,4096,8192,16384};
        for (int i = 0; i < (progs[4]).BandsActive; i++)
            (progs[4]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[4]).name, "Octaves (10 Bands)", kMaxLen);
    }
    {
        //program 6: F:[22.474,33.711,50.567,75.851,113.777,170.666,256,384,
        //              576,864,1296,1944,2916,4374,6561,9841.5,14762.25]
        (progs[5]).BandsActive = 17;
        (progs[5]).BWlink = true;                   //BWs[] will be ignored
        (progs[5]).BW = .2f;

        float fr[] = {22.474f,33.711f,50.567f,75.851f,113.777f,170.666f,256,384,
                      576,864,1296,1944,2916,4374,6561,9841.5f,14762.25f};
        for (int i = 0; i < (progs[5]).BandsActive; i++)
            (progs[5]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[5]).name, "Fifths (17 Bands)", kMaxLen);
    }
    {
        //program 7: F:[25.628,34.171,45.562,60.75,81,108,144,192,256,341.333,455.11,606.814,
        //             809.086,1078.781,1438.375,1917.834,2557.112,3409.483,4545.978,6061.304,
        //             8081.738,10775.652,14367.535,19156.714]
        (progs[6]).BandsActive = 24;
        (progs[6]).BWlink = true;                   //BWs[] will be ignored
        (progs[6]).BW = .2f;

        float fr[] = {25.628f,34.171f,45.562f,60.75f,81,108,144,192,256,341.333f,455.11f,606.814f,
                      809.086f,1078.781f,1438.375f,1917.834f,2557.112f,3409.483f,4545.978f,6061.304f,
                      8081.738f,10775.652f,14367.535f,19156.714f};
        for (int i = 0; i < (progs[6]).BandsActive; i++)
            (progs[6]).freqs[i] = fr[i];
        
        vst_strncpy ((progs[6]).name, "Thirds (24 Bands)", kMaxLen);
    }
    
    //now load the defaults into all parameters
    setProgram(0);
    for (int i = 0; i < kNumBands; i++)
    {
        setParameter (kBW1 + i, (progs[0]).BW);
        setParameter (kBand1 + i, 0.5f);
    }
    setParameter (kFreq32, 1.0f);
    setParameter (kGain, 0.5f);
}

//-----------------------------------------------------------------------------
void PWSEQ::setProgram (VstInt32 program)
{
    if (program < 0 || program >= kNumPrograms) return;
	settingProgram = true;
    
    curProg = program;
    
    setParameter (kNumBandsActive, (float) (progs[program]).BandsActive / (float) kNumBands);
    setParameter (kLinkBW, ((progs[program]).BWlink ? 1.0f : 0.0f));
    setParameter (kGain, (progs[program]).mainGain / dbRange / 2 + 0.5f);
	setParameter (kMinFreq, (progs[program]).freqMin / fs * 2.0f);
	setParameter (kMaxFreq, (progs[program]).freqMax / fs * 2.0f);
    if ((progs[program]).BWlink)
        setParameter (kBW, (progs[program]).BW);
    else
    {
        for (int i = 0; i < (progs[program]).BandsActive; i++)
            setParameter (kBW1 + i, (progs[program]).BWs[i]);
    }
    
    for (int i = 0; i < (progs[program]).BandsActive; i++)
	{
        setParameter (kFreq1 + i, (progs[program]).freqs[i] / fs * 2.0f);
                    //spread them between 0 and 1 (instead of 0.5)
		setParameter (kBand1 + i, (progs[program]).bandGain[i] / dbRange / 2 + 0.5f);
					//change interval [-12,12] to [0,1]
	}

    if (EditorLoaded)
    {
        for (int i = 0; i < kNumBands; i++)
        {
            ((AEffGUIEditor*)editor)->setParameter (kBW1 + i, 0.0f); //value is ignored
            ((AEffGUIEditor*)editor)->setParameter (kFreq1 + i, 0.0f);
			((AEffGUIEditor*)editor)->setParameter (kBand1 + i, 0.0f);
        }
        ((AEffGUIEditor*)editor)->setParameter (kGain, 0.0f);
        ((AEffGUIEditor*)editor)->setParameter (kLinkBW, 0.0f);
        ((AEffGUIEditor*)editor)->setParameter (kBW, 0.0f);
        ((AEffGUIEditor*)editor)->setParameter (kNumBandsActive, 0.0f);
		((AEffGUIEditor*)editor)->setParameter (kGain, 0.0f);
    }
	settingProgram = false;
}

//-----------------------------------------------------------------------------
void PWSEQ::setProgramName (char* name)
{
    vst_strncpy ((progs[curProg]).name, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------
VstInt32 PWSEQ::getProgram ()
{
    return curProg;
}

//-----------------------------------------------------------------------------
void PWSEQ::getProgramName (char* name)
{
    vst_strncpy (name, (progs[curProg]).name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------
bool PWSEQ::getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text)
{
    if (index < 0 || index >= kNumPrograms) return false;
    
    vst_strncpy (text, (progs[index]).name, kVstMaxProgNameLen);
    return true;
}

//-----------------------------------------------------------------------------
// 7. Midi
//-----------------------------------------------------------------------------
VstInt32 PWSEQ::getNumMidiInputChannels ()
{
    return kNumMidiInputs;
}

//-----------------------------------------------------------------------------
VstInt32 PWSEQ::getNumMidiOutputChannels ()
{
    return kNumMidiOutputs;
}

//-----------------------------------------------------------------------------
VstInt32 PWSEQ::processEvents (VstEvents* events)
{
    ev = events;  //events will be checked in checkEvents()
    
    return 0;     //return value is ignored
}

//-----------------------------------------------------------------------------
void PWSEQ::checkEvents (int num)
{
    if (ev == NULL) return;
    int numEvents = ev->numEvents;        //the number of events
    VstEvent** events = ev->events;        //array of events
    
    for (int i = 0; i < numEvents; i++)
    {
        if ((events[i])->type == kVstMidiType)
        {
            VstMidiEvent* midiEv = (VstMidiEvent*) events[i];
            
            //do something with the deltaFrames
            
            char evtype = midiEv->midiData[0];
            if ((evtype & 0xB0) == 0xB0) //controller event
            {
                //7  (volume)
                if (midiEv->deltaFrames == num && midiEv->midiData[1] == 7)
                {
                    setParameter (kGain, midiEv->midiData[2] / 127.0f);
                }
                
                //80 (general purpose button 1: bypass)
                if (midiEv->deltaFrames == num && midiEv->midiData[1] == 80)
                {
                    bool bp = true;
                    if (midiEv->midiData[2] < 64) bp = false;
                    
                    setBypass (bp);
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------
// 8. Main process
//-----------------------------------------------------------------------------
void PWSEQ::processReplacing(float ** inputs, float** outputs, VstInt32 sampleFrames)
{
    float* in1 = inputs[0];
    float* in2 = inputs[1];
    float* out1 = outputs[0];
    float* out2 = outputs[1];
    
    int i = 0;
    
    while (sampleFrames--)
    {
        //checkEvents(i++);
        if (!ByPass)
        {
            (*out1++) = EQ->processL(gain * (*in1++));
            (*out2++) = EQ->processR(gain * (*in2++));
        }
        else
        {
            (*out1++) = (*in1++);
            (*out2++) = (*in2++);
        }
    }
    ev = NULL;           //just in case there are no events in the next block
}

//-----------------------------------------------------------------------------
void PWSEQ::processDoubleReplacing(double ** inputs, double** outputs, VstInt32 sampleFrames)
{
    double* in1 = inputs[0];
    double* in2 = inputs[1];
    double* out1 = outputs[0];
    double* out2 = outputs[1];
    
    int i = 0;
    
    while (sampleFrames--)
    {
        //checkEvents(i++);
        if (!ByPass)
        {
            (*out1++) = (double) EQ->processL(gain * (float) (*in1++));
            (*out2++) = (double) EQ->processR(gain * (float) (*in2++));
        }
        else
        {
            (*out1++) = (*in1++);
            (*out2++) = (*in2++);
        }
    }
    ev = NULL;
}
