//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* peaking.cpp
 * 
 * Creates a biquad peaking filter (2 poles, 2 zeros).
 * You can use peakinghandler.cpp to handle multiple peaking filters in series.
 * That way it creates an equalizer.
 * 
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 */

#include "peaking.h"

//-----------------------------------------------------------------------------
// Constructor and destructor
//-----------------------------------------------------------------------------
peaking::peaking()
{
    f = 0.0f;
    b = 0.0f;
    p = 0.0f;
    
    fSet = false;
    bSet = false;
    pSet = false;
    canfilter = false;
    
    xl1 = xl2 = 0.0f;
    yl1 = yl2 = 0.0f;
    xr1 = xr2 = 0.0f;
    yr1 = yr2 = 0.0f;

    pi = 3.14159265f;
}

//-----------------------------------------------------------------------------
peaking::peaking(float freq, float BW, float peak)
{
    f = freq;
    b = BW;
    p = peak;

    pi = 3.14159265f;
    
    c = cos(2.0f * pi * f);
    
    fSet = true;
    bSet = true;
    pSet = true;
    
    xl1 = xl2 = 0.0f;
    yl1 = yl2 = 0.0f;
    xr1 = xr2 = 0.0f;
    yr1 = yr2 = 0.0f;
    
    calc();
}

//-----------------------------------------------------------------------------
peaking::~peaking()
{
    //delete all dynamic variables
}

//-----------------------------------------------------------------------------
// peakingfilter() functions
//-----------------------------------------------------------------------------
float peaking::peakingfilterL(float in)
{
    if (canfilter)
    {
        float out = in + a1 * xl1 + a2 * xl2 + b1 * yl1 + b2 * yl2;
        xl2 = xl1;
        xl1 = in;
        yl2 = yl1;
        yl1 = out;
        return out * gain;
    }
    else return 0.0f;
}

//-----------------------------------------------------------------------------
float peaking::peakingfilterR(float in)
{
    if (canfilter)
    {
        float out = in + a1 * xr1 + a2 * xr2 + b1 * yr1 + b2 * yr2;
        xr2 = xr1;
        xr1 = in;
        yr2 = yr1;
        yr1 = out;
        return out * gain;
    }
    else return 0.0f;
}

//-----------------------------------------------------------------------------
// H(z)
//-----------------------------------------------------------------------------
float peaking::H(float z)
{
    return (z * z + a1 * z + a2) / (z * z - b1 * z - b2);
}

//-----------------------------------------------------------------------------
// Turning off and on
//-----------------------------------------------------------------------------
void peaking::turnOff()
{
    canfilter = false;
}

//-----------------------------------------------------------------------------
void peaking::turnOn()
{
    canfilter = true;
}

//-----------------------------------------------------------------------------
// calc() and settings-changing functions
//-----------------------------------------------------------------------------
void peaking::calc()
{
    absz = 1.0f - b*f*p;
    absp = 1.0f - b*f;
    
    a1 = -2 * absz * c;
    a2 = absz * absz;
    b1 = 2 * absp * c;
    b2 = - absp * absp;
    
    if (p > 1.0f)
    {
        gain = 1.0f / min(H(1.0f), H(-1.0f)); //take the minimum...
    }
    else
    {
        gain = 1.0f / max(H(1.0f), H(-1.0f)); //or the maximum...
    }
    //...of the frequency responses at f=0 and f=0.5 (z=1 and z=-1)
    
    canfilter = true;
}

//-----------------------------------------------------------------------------
void peaking::setFreq(float freq)
{
    f = freq;
    c = cos(2.0f * pi * f);
    fSet = true;
    if (bSet && pSet) calc();
}

float peaking::getFreq()
{
    return f;
}

//-----------------------------------------------------------------------------
void peaking::setBW(float BW)
{
    b = BW;
    bSet = true;
    if (fSet && pSet) calc();
}

float peaking::getBW()
{
    return b;
}

//-----------------------------------------------------------------------------
void peaking::setPeak(float peak)
{
    p = peak;
    pSet = true;
    if (fSet && bSet) calc();
}

float peaking::getPeak()
{
    return p;
}
