//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* peakinghandler.cpp
 * 
 * Contains a handler for peaking filters.
 * It handles multiple peaking filters in series.
 * This is a great tool for making parametric equalizers, but you can use it for
 * other kinds of filters as well.
 * 
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 */

#include "peakinghandler.h"
#include <assert.h>
#include <stdio.h>

//-----------------------------------------------------------------------------
// Constructor and destructor
//-----------------------------------------------------------------------------
peakinghandler::peakinghandler(int num, float* freqs, float* BWs, float* peaks)
{
    n = num;
    nA = n;
    f = freqs;
    b = BWs;
    p = peaks;
    
    filters = new peaking[n];
    assert(filters != NULL);
    
    for (int i = 0; i < n; i++)
    {
        (filters[i]).setFreq(f[i]);
        (filters[i]).setBW(b[i]);
        (filters[i]).setPeak(p[i]);
    }
    
    linked = false;
}

//-----------------------------------------------------------------------------
peakinghandler::~peakinghandler()
{
    if (filters != NULL) delete[] filters;
}

//-----------------------------------------------------------------------------
// process() functions
//-----------------------------------------------------------------------------
float peakinghandler::processL(float in)
{
    float out = in;
    int i;
    for (i = 0; i < nA; i++)
    {
        out = (filters[i]).peakingfilterL(out);
    }
    for (/*nothing required here*/ ;i < n; i++)
    {
        //(filters[i]).peakingfilterL(0.0f);     //make shure no samples are being kept for recursion
    }
    return out;
}

//-----------------------------------------------------------------------------
float peakinghandler::processR(float in)
{
    float out = in;
    int i;
    for (i = 0; i < nA; i++)
    {
        out = (filters[i]).peakingfilterR(out);
    }
    for (/*nothing required here*/ ;i < n; i++)
    {
        //(filters[i]).peakingfilterR(0.0f);     //make shure no samples are being kept for recursion
    }
    return out;
}

//-----------------------------------------------------------------------------
// Settings-changing functions
//-----------------------------------------------------------------------------
void peakinghandler::linkBW(bool l)
{
    if (linked != l)
    {
        linked = l;
        for (int i = 0; i < n; i++)
        {
            if (linked) (filters[i]).setBW(bAll);
            else (filters[i]).setBW(b[i]);
        }
    }
}

//-----------------------------------------------------------------------------
bool peakinghandler::BWlinked()
{
    return linked;
}

//-----------------------------------------------------------------------------
void peakinghandler::setBW(float BW)
{
    bAll = BW;
    if (linked)
    {
        for (int i = 0; i < n; i++)
        {
            (filters[i]).setBW(bAll);
        }
    }
}

//-----------------------------------------------------------------------------
float peakinghandler::getBW()
{
    return bAll;
}

//-----------------------------------------------------------------------------
void peakinghandler::setNumActive(int num)
{
    if (num > nA) //some filters have been turned on again
    {
        for (int i = nA; i < num; i++)
        {
            (filters[i]).turnOn();
        }
    }
    else if (num < nA) //some filters have been turned off
    {
        for (int i = num; i < nA; i++)
        {
            (filters[i]).turnOff();
        }
    }
    nA = num;    
}

//-----------------------------------------------------------------------------
int peakinghandler::getNumActive()
{
    return nA;
}

//-----------------------------------------------------------------------------
void peakinghandler::freqChanged(int num)
{
    (filters[num]).setFreq(f[num]);
}

//-----------------------------------------------------------------------------
void peakinghandler::BWChanged(int num)
{
    if (!linked) (filters[num]).setBW(b[num]);
}

//-----------------------------------------------------------------------------
void peakinghandler::peakChanged(int num)
{
    (filters[num]).setPeak(p[num]);
}

//-----------------------------------------------------------------------------
// logSpread()
// Spread the frequencies logarithmically between fMin and fMax
//-----------------------------------------------------------------------------
void peakinghandler::logSpread(float fMin, float fMax)
{
    float g = pow(fMax/fMin, 1.0f / (nA - 1)); //g must be the nth root of fMax/fMin with n = nA - 1
    for (int i = 0; i < nA; i++)
    {
        f[i] = fMin * pow(g, (float) i); //so the first frequency equals fMin and the last one
                                         //equals fMin * g^(nA - 1) = fMax
        freqChanged(i);
    }
}
